﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="TargetManager.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the <seealso cref="TargetManager" /> type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Business
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.Contracts;
    using System.Linq;

    using Hisaab.Business.Entities;
    using Hisaab.Business.Interfaces;
    using Hisaab.Common;
    using Hisaab.Data.Interfaces;

    using Microsoft.Practices.ServiceLocation;

    /// <summary>
    ///  Defines the <seealso cref="TargetManager"/> type.
    /// </summary>
    public class TargetManager : ITargetManager
    {
        /// <summary>
        /// Member Field _targetRepository
        /// </summary>
        private readonly ITargetRepository _targetRepository;

        /// <summary>
        /// Initializes a new instance of the <see cref="TargetManager"/> class.
        /// </summary>
        public TargetManager()
            : this(ServiceLocator.Current.GetInstance<ITargetRepository>())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TargetManager"/> class.
        /// </summary>
        /// <param name="targetRepository">The target repository.</param>
        public TargetManager(ITargetRepository targetRepository)
        {
            Contract.Requires<ArgumentNullException>(targetRepository != null);

            _targetRepository = targetRepository;
        }

        #region Implementation of IManagerBase<Target>

        /// <summary>
        /// Gets the filtered.
        /// </summary>
        /// <param name="filter">The filter.</param>
        /// <param name="pageNo">The page no.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <returns>List of filtered items</returns>
        public IEnumerable<Target> List(Target filter = null, int? pageNo = null, int? pageSize = null)
        {
            return _targetRepository.Query(filter.Map<Target, Data.Entities.Target>(), pageNo, pageSize)
                .ToList()
                .Select(x => x.Map<Data.Entities.Target, Target>());
        }

        /// <summary>
        /// Adds the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        /// <returns>The id of entity being added.</returns>
        public int Add(Target entity)
        {
            return _targetRepository.Add(entity.Map<Target, Data.Entities.Target>());
        }

        /// <summary>
        /// Updates the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        public void Update(Target entity)
        {
            _targetRepository.Update(entity.Map<Target, Data.Entities.Target>());
        }

        /// <summary>
        /// Deletes the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        public void Delete(Target entity)
        {
            _targetRepository.Delete(entity.Map<Target, Data.Entities.Target>());
        }

        #endregion Implementation of IManagerBase<Target>

        /// <summary>
        /// Gets the by id.
        /// </summary>
        /// <param name="id">The id of target</param>
        /// <returns>Target matching by id</returns>
        public Target GetById(int id)
        {
            return _targetRepository.FindById(id).Map<Data.Entities.Target, Target>();
        }

        /// <summary>
        /// Gets the name of the by.
        /// </summary>
        /// <param name="targetName">Name of the target.</param>
        /// <returns>Target matching by name</returns>
        public Target GetByName(string targetName)
        {
            return _targetRepository.Query(new Data.Entities.Target { Title = targetName })
                  .FirstOrDefault()
                  .Map<Data.Entities.Target, Target>();
        }
    }
}