// --------------------------------------------------------------------------------------------------------------------
// <copyright file="BackstageMergeRegionAdapter.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Enables use of a Ribbon control as a Prism region.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Client.Common.Adapters
{
    using System.Collections.Specialized;
    using System.Windows;

    using Fluent;

    using Microsoft.Practices.Prism.Regions;

    /// <summary>
    /// Enables use of a Ribbon control as a Prism region.
    /// </summary>
    /// <remarks> See Developer's Guide to Microsoft Prism (Ver. 4), p. 189.</remarks>
    public class BackstageMergeRegionAdapter : RegionAdapterBase<BackstageTabControl>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="BackstageMergeRegionAdapter"/> class.
        /// </summary>
        /// <param name="behaviorFactory">
        /// Allows the registration of the default set of RegionBehaviors.
        /// </param>
        public BackstageMergeRegionAdapter(IRegionBehaviorFactory behaviorFactory)
            : base(behaviorFactory)
        {
        }

        /// <summary>
        /// Adapts a WPF control to serve as a Prism IRegion.
        /// </summary>
        /// <param name="region">The new region being used.</param>
        /// <param name="regionTarget">The WPF control to adapt.</param>
        protected override void Adapt(IRegion region, BackstageTabControl regionTarget)
        {
            region.Views.CollectionChanged += (sender, e) =>
            {
                switch (e.Action)
                {
                    case NotifyCollectionChangedAction.Add:
                        foreach (FrameworkElement element in e.NewItems)
                        {
                            var backstageTabControl = element as BackstageTabControl;
                            if (backstageTabControl != null)
                            {
                                int lastPosition = regionTarget.Items.Count;
                                for (int j = backstageTabControl.Items.Count - 1; j >= 0; j--)
                                {
                                    var tabItem = backstageTabControl.Items[j] as BackstageTabItem;
                                    backstageTabControl.Items.RemoveAt(j);
                                    regionTarget.Items.Insert(lastPosition, tabItem);
                                }
                            }
                        }

                        break;

                    case NotifyCollectionChangedAction.Remove:
                        foreach (UIElement elementLoopVariable in e.OldItems)
                        {
                            var element = elementLoopVariable;
                            var group = element as RibbonGroupBox;
                            if (group != null)
                            {
                                if (regionTarget.Items.Contains(element as RibbonGroupBox))
                                {
                                    regionTarget.Items.Remove(element as RibbonGroupBox);
                                }
                            }
                        }

                        break;
                }
            };
        }

        /// <summary>
        /// Creates the region.
        /// </summary>
        /// <returns>Region object</returns>
        protected override IRegion CreateRegion()
        {
            return new AllActiveRegion();
        }
    }
}