// --------------------------------------------------------------------------------------------------------------------
// <copyright file="ViewModelBase.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the <seealso cref="ViewModelBase" /> type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Client.Common.BaseClasses
{
    using System.ComponentModel;
    using System.Windows;
    using System.Windows.Input;
    using System.Windows.Media;

    /// <summary>
    /// Defines the <seealso cref="ViewModelBase"/> type.
    /// </summary>
    public abstract class ViewModelBase : DependencyObject, INotifyPropertyChanging, INotifyPropertyChanged
    {
        /// <summary>
        /// Using a DependencyProperty as the backing store for Title.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty TitleProperty = DependencyProperty.Register("Title", typeof(string), typeof(TabViewModel));

        /// <summary>
        /// Member variable _image
        /// </summary>
        private ImageSource _image;

        /// <summary>
        /// Member variable _imageSmall
        /// </summary>
        private ImageSource _imageSmall;

        #region INotifyPropertyChanging Members

        /// <summary>
        /// Occurs when a property value is changing.
        /// </summary>
        public event PropertyChangingEventHandler PropertyChanging;

        #endregion INotifyPropertyChanging Members

        #region INotifyPropertyChanged Members

        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion INotifyPropertyChanged Members

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>
        /// The title.
        /// </value>
        public string Title
        {
            get { return (string)GetValue(TitleProperty); }
            set { SetValue(TitleProperty, value); }
        }

        /// <summary>
        /// Gets or sets the image.
        /// </summary>
        /// <value>
        /// The image.
        /// </value>
        public ImageSource Image
        {
            get
            {
                return _image;
            }

            set
            {
                _image = value;
                RaisePropertyChangedEvent("Image");
            }
        }

        /// <summary>
        /// Gets or sets the image.
        /// </summary>
        /// <value>
        /// The image.
        /// </value>
        public ImageSource ImageSmall
        {
            get
            {
                return _imageSmall;
            }

            set
            {
                _imageSmall = value;
                RaisePropertyChangedEvent("ImageSmall");
            }
        }

        #region Administrative Properties

        /// <summary>
        /// Gets or sets a value indicating whether [ignore property change events].
        /// </summary>
        /// <value>
        /// <c>true</c> if [ignore property change events]; otherwise, <c>false</c>.
        /// </value>
        public virtual bool IgnorePropertyChangeEvents { get; set; }

        #endregion Administrative Properties

        #region Public Methods

        /// <summary>
        /// Raises the PropertyChanged event.
        /// </summary>
        /// <param name="propertyName">The name of the changed property.</param>
        public virtual void RaisePropertyChangedEvent(string propertyName)
        {
            if (IgnorePropertyChangeEvents)
            {
                return;
            }

            if (PropertyChanged == null)
            {
                return;
            }

            var e = new PropertyChangedEventArgs(propertyName);
            PropertyChanged(this, e);
            CommandManager.InvalidateRequerySuggested();
        }

        /// <summary>
        /// Raises the PropertyChanging event.
        /// </summary>
        /// <param name="propertyName">The name of the changing property.</param>
        public virtual void RaisePropertyChangingEvent(string propertyName)
        {
            if (IgnorePropertyChangeEvents)
            {
                return;
            }

            if (PropertyChanging == null)
            {
                return;
            }

            var e = new PropertyChangingEventArgs(propertyName);
            PropertyChanging(this, e);
        }

        /// <summary>
        /// Closes the view.
        /// </summary>
        public virtual void CloseView()
        {
        }

        #endregion Public Methods
    }
}