// --------------------------------------------------------------------------------------------------------------------
// <copyright file="AccountSideBarViewModel.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the <seealso cref="AccountSideBarViewModel" /> type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Client.Modules.Core.Account.ViewModel
{
    using System;
    using System.Collections.ObjectModel;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Windows.Input;

    using Hisaab.Business.Interfaces;
    using Hisaab.Client.Common;
    using Hisaab.Client.Common.BaseClasses;
    using Hisaab.Client.Common.Services;
    using Hisaab.Client.Model;
    using Hisaab.Common;

    using Microsoft.Practices.Prism.Commands;
    using Microsoft.Practices.ServiceLocation;

    /// <summary>
    /// Defines the <seealso cref="AccountSideBarViewModel"/> type.
    /// </summary>
    public class AccountSideBarViewModel : ViewModelBase
    {
        /// <summary>
        /// Session manager
        /// </summary>
        private readonly ISession _session;

        /// <summary>
        /// Account manager
        /// </summary>
        private readonly IAccountManager _accountManager;

        /// <summary>
        /// Backing field for account total
        /// </summary>
        private ObservableCollection<KeyValue<Account, decimal>> _accountTotals;

        /// <summary>
        /// Member variable _refreshCommand
        /// </summary>
        private ICommand _refreshCommand;

        /// <summary>
        /// Initializes a new instance of the <see cref="AccountSideBarViewModel"/> class.
        /// </summary>
        public AccountSideBarViewModel()
            : this(
            ServiceLocator.Current.GetInstance<IAccountManager>(),
            ServiceLocator.Current.GetInstance<ISession>())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AccountSideBarViewModel"/> class.
        /// </summary>
        /// <param name="accountManager">The account manager.</param>
        /// <param name="session">The Session</param>
        public AccountSideBarViewModel(IAccountManager accountManager, ISession session)
        {
            Contract.Requires<ArgumentNullException>(accountManager != null);
            Contract.Requires<ArgumentNullException>(session != null);

            Title = "Account Summary";

            _accountManager = accountManager;
            _session = session;

            GlobalCommands.Refresh.RegisterCommand(RefreshCommand);

            Refresh();
        }

        /// <summary>
        /// Gets the refresh command.
        /// </summary>
        /// <value>
        /// The refresh.
        /// </value>
        public ICommand RefreshCommand
        {
            get
            {
                if (_refreshCommand == null)
                {
                    _refreshCommand = new DelegateCommand(Refresh);
                }

                return _refreshCommand;
            }
        }

        /// <summary>
        /// Gets or sets the account totals.
        /// </summary>
        /// <value>The account totals.</value>
        public ObservableCollection<KeyValue<Account, decimal>> AccountTotals
        {
            get
            {
                return _accountTotals;
            }

            set
            {
                _accountTotals = value;
                RaisePropertyChangedEvent("AccountTotals");
            }
        }

        /// <summary>
        /// Refreshes this data.
        /// </summary>
        public void Refresh()
        {
            var totals = _accountManager.GetAccountTotals(_session.LoggedUser).Select(x => new KeyValue<Account, decimal> { Key = x.Key.Map<Business.Entities.Account, Account>(), Value = x.Value });
            AccountTotals = new ObservableCollection<KeyValue<Account, decimal>>(totals);
        }
    }
}