// --------------------------------------------------------------------------------------------------------------------
// <copyright file="TargetEditorViewModel.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the <seealso cref="TargetEditorViewModel" /> type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Client.Modules.Core.Target.ViewModel
{
    using System;
    using System.Collections.ObjectModel;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Windows.Input;
    using System.Windows.Media.Imaging;

    using Hisaab.Business.Interfaces;
    using Hisaab.Client.Common.BaseClasses;
    using Hisaab.Common;

    using Microsoft.Practices.Prism.Commands;
    using Microsoft.Practices.Prism.Regions;
    using Microsoft.Practices.ServiceLocation;

    /// <summary>
    /// Defines the <seealso cref="TargetEditorViewModel"/> type.
    /// </summary>
    public class TargetEditorViewModel : WindowViewModel, INavigationAware
    {
        /// <summary>
        /// Member varibale target manager
        /// </summary>
        private readonly ITargetManager _targetManager;

        /// <summary>
        /// Member varibale tag manager
        /// </summary>
        private readonly ITagManager _tagManager;

        /// <summary>
        /// Member variable account manager
        /// </summary>
        private readonly IAccountManager _accountManager;

        /// <summary>
        /// Selected target
        /// </summary>
        private Model.Target _selectedTarget;

        /// <summary>
        /// Member variable _saveCommand
        /// </summary>
        private DelegateCommand _saveCommand;

        /// <summary>
        /// Member variable tags
        /// </summary>
        private ObservableCollection<Model.Tag> _tags;

        /// <summary>
        /// Member variable accounts
        /// </summary>
        private ObservableCollection<Model.Account> _accounts;

        /// <summary>
        /// Initializes a new instance of the <see cref="TargetEditorViewModel"/> class.
        /// </summary>
        public TargetEditorViewModel()
            : this(
            ServiceLocator.Current.GetInstance<ITargetManager>(),
            ServiceLocator.Current.GetInstance<ITagManager>(),
            ServiceLocator.Current.GetInstance<IAccountManager>())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TargetEditorViewModel"/> class.
        /// </summary>
        /// <param name="targetManager">The target Manager.</param>
        /// <param name="tagManager">The tag Manager.</param>
        /// <param name="accountManager">The account Manager.</param>
        public TargetEditorViewModel(ITargetManager targetManager, ITagManager tagManager, IAccountManager accountManager)
        {
            Contract.Requires<ArgumentNullException>(targetManager != null);
            Contract.Requires<ArgumentNullException>(tagManager != null);
            Contract.Requires<ArgumentNullException>(accountManager != null);

            _targetManager = targetManager;
            _tagManager = tagManager;
            _accountManager = accountManager;

            Title = "Target Editor";
            Icon = new BitmapImage(new Uri("pack://application:,,,/Hisaab.Client.Common;component/Resources/Images/16/target.png"));
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is new target.
        /// </summary>
        /// <value>
        /// True if this instance is new target; otherwise, false.
        /// </value>
        public bool IsNewTarget { get; set; }

        /// <summary>
        /// Gets the dialog type string.
        /// </summary>
        /// <value>
        /// The dialog type string.
        /// </value>
        public string DialogTypeString
        {
            get
            {
                return IsNewTarget ? "Add" : "Edit";
            }
        }

        /// <summary>
        /// Gets or sets the selected target.
        /// </summary>
        /// <value>The selected target.</value>
        public Model.Target SelectedTarget
        {
            get
            {
                return _selectedTarget;
            }

            set
            {
                _selectedTarget = value;
                RaisePropertyChangedEvent("SelectedTarget");
            }
        }

        /// <summary>
        /// Gets the tags.
        /// </summary>
        /// <value>The tags.</value>
        public ObservableCollection<Model.Tag> Tags
        {
            get
            {
                if (_tags == null)
                {
                    _tags = new ObservableCollection<Model.Tag>(_tagManager.List().Select(x => x.Map<Business.Entities.Tag, Model.Tag>()));
                }

                return _tags;
            }
        }

        /// <summary>
        /// Gets the accounts.
        /// </summary>
        /// <value>The accounts.</value>
        public ObservableCollection<Model.Account> Accounts
        {
            get
            {
                if (_accounts == null)
                {
                    _accounts = new ObservableCollection<Model.Account>(_accountManager.List().Select(x => x.Map<Business.Entities.Account, Model.Account>()));
                }

                return _accounts;
            }
        }

        /// <summary>
        /// Gets the save command.
        /// </summary>
        public ICommand SaveCommand
        {
            get
            {
                if (_saveCommand == null)
                {
                    _saveCommand = new DelegateCommand(Save);
                }

                return _saveCommand;
            }
        }

        #region Member Methods

        /// <summary>
        /// Saves this instance.
        /// </summary>
        public void Save()
        {
            if (!string.IsNullOrWhiteSpace(SelectedTarget.Error))
            {
                return;
            }

            if (IsNewTarget)
            {
                _targetManager.Add(SelectedTarget.Map<Model.Target, Business.Entities.Target>());
            }
            else
            {
                _targetManager.Update(SelectedTarget.Map<Model.Target, Business.Entities.Target>());
            }
        }

        #endregion Member Methods

        #region Implementation of INavigationAware

        /// <summary>
        /// Called when [navigated to].
        /// </summary>
        /// <param name="navigationContext">The navigation context.</param>
        public void OnNavigatedTo(NavigationContext navigationContext)
        {
            if (navigationContext.Parameters["ID"] != null)
            {
                IsNewTarget = false;
                int id = int.Parse(navigationContext.Parameters["ID"]);
                SelectedTarget = _targetManager.GetById(id).Map<Business.Entities.Target, Model.Target>();
                SelectedTarget.Dirty = false;
            }
            else
            {
                IsNewTarget = true;
                SelectedTarget = new Model.Target { Dirty = true, TargetDate = DateTime.Today };
            }
        }

        /// <summary>
        /// Determines whether [is navigation target] [the specified navigation context].
        /// </summary>
        /// <param name="navigationContext">The navigation context.</param>
        /// <returns>
        ///   <c>true</c> if [is navigation target] [the specified navigation context]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            return false;
        }

        /// <summary>
        /// Called when [navigated from].
        /// </summary>
        /// <param name="navigationContext">The navigation context.</param>
        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
        }

        #endregion Implementation of INavigationAware
    }
}