﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="TargetNavigationViewModel.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the <seealso cref="TargetNavigationViewModel" /> type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Client.Modules.Core.Target.ViewModel
{
    using System;
    using System.Collections.ObjectModel;
    using System.Diagnostics.Contracts;
    using System.Globalization;
    using System.Linq;
    using System.Windows.Input;
    using System.Windows.Media.Imaging;

    using Hisaab.Business.Interfaces;
    using Hisaab.Client.Common;
    using Hisaab.Client.Common.BaseClasses;
    using Hisaab.Common;

    using Microsoft.Practices.Prism;
    using Microsoft.Practices.Prism.Commands;
    using Microsoft.Practices.Prism.Regions;
    using Microsoft.Practices.ServiceLocation;

    /// <summary>
    /// Defines the <seealso cref="TargetNavigationViewModel"/> type.
    /// </summary>
    public class TargetNavigationViewModel : NavigationViewModel
    {
        #region Data Members

        /// <summary>
        /// Member variable _targetManager
        /// </summary>
        private readonly ITargetManager _targetManager;

        /// <summary>
        /// Member variable _targets
        /// </summary>
        private ObservableCollection<Model.Target> _targets;

        /// <summary>
        /// Member variable _selectedTarget
        /// </summary>
        private Model.Target _selectedTarget;

        /// <summary>
        /// Member variable _refreshCommand
        /// </summary>
        private ICommand _refreshCommand;

        #endregion Data Members

        /// <summary>
        /// Initializes a new instance of the <see cref="TargetNavigationViewModel"/> class.
        /// </summary>
        public TargetNavigationViewModel()
            : this(ServiceLocator.Current.GetInstance<ITargetManager>())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TargetNavigationViewModel"/> class.
        /// </summary>
        /// <param name="targetManager">The target manager.</param>
        public TargetNavigationViewModel(ITargetManager targetManager)
        {
            Contract.Requires<ArgumentNullException>(targetManager != null);
            _targetManager = targetManager;

            Title = "Targets";
            Image = new BitmapImage(
                        new Uri(
                        "Pack://application:,,,/Hisaab.Client.Common;component/Resources/Images/32/Target.png",
                        UriKind.Absolute));

            ImageSmall =
                        new BitmapImage(
                        new Uri(
                        "Pack://application:,,,/Hisaab.Client.Common;component/Resources/Images/16/Target.png",
                        UriKind.Absolute));

            GlobalCommands.Refresh.RegisterCommand(RefreshCommand);
            RefreshList();
        }

        /// <summary>
        /// Gets or sets the targets.
        /// </summary>
        /// <value>The targets.</value>
        public ObservableCollection<Model.Target> Targets
        {
            get
            {
                return _targets;
            }

            set
            {
                RaisePropertyChangingEvent("Targets");
                _targets = value;
                RaisePropertyChangedEvent("Targets");
            }
        }

        /// <summary>
        /// Gets the refresh command.
        /// </summary>
        /// <value>
        /// The refresh command.
        /// </value>
        public ICommand RefreshCommand
        {
            get
            {
                if (_refreshCommand == null)
                {
                    _refreshCommand = new DelegateCommand(RefreshList);
                }

                return _refreshCommand;
            }
        }

        /// <summary>
        /// Gets or sets the selected target.
        /// </summary>
        /// <value>The selected target.</value>
        public Model.Target SelectedTarget
        {
            get
            {
                return _selectedTarget;
            }

            set
            {
                RaisePropertyChangingEvent("SelectedTarget");
                _selectedTarget = value;
                RaisePropertyChangedEvent("SelectedTarget");
                OnTargetChanged();
            }
        }

        /// <summary>
        /// Called when target changed.
        /// </summary>
        public void OnTargetChanged()
        {
            if (SelectedTarget != null)
            {
                // Initialize
                var regionManager = ServiceLocator.Current.GetInstance<IRegionManager>();

                // Show Workspace
                var query = new UriQuery { { "ID", SelectedTarget.Id.ToString(CultureInfo.InvariantCulture) } };
                var dashboardUri = new Uri(TargetModule.TagetView + query, UriKind.RelativeOrAbsolute);
                regionManager.RequestNavigate(Regions.WorkspaceRegion, dashboardUri);

                LastNavigatedUri = dashboardUri;
            }
        }

        /// <summary>
        /// Refreshes the list.
        /// </summary>
        public void RefreshList()
        {
            Targets = new ObservableCollection<Model.Target>(
                _targetManager.List()
                .Select(x => x.Map<Business.Entities.Target, Model.Target>()));
        }
    }
}