// --------------------------------------------------------------------------------------------------------------------
// <copyright file="TargetsViewModel.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the <seealso cref="TargetsViewModel" /> type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Client.Modules.Core.Target.ViewModel
{
    using System;
    using System.Collections.ObjectModel;
    using System.Diagnostics.Contracts;
    using System.Globalization;
    using System.Linq;
    using System.Windows;
    using System.Windows.Input;

    using Hisaab.Business.Interfaces;
    using Hisaab.Client.Common;
    using Hisaab.Client.Common.BaseClasses;
    using Hisaab.Client.Common.Services;
    using Hisaab.Client.Modules.Core.Properties;
    using Hisaab.Client.Modules.Core.Target.Commands;
    using Hisaab.Common;

    using Microsoft.Practices.Prism;
    using Microsoft.Practices.Prism.Commands;
    using Microsoft.Practices.Prism.Regions;
    using Microsoft.Practices.ServiceLocation;

    /// <summary>
    /// Defines the <seealso cref="TargetsViewModel"/> type.
    /// </summary>
    public class TargetsViewModel : TabViewModel
    {
        #region Data Members

        /// <summary>
        /// Member variable _targetManager
        /// </summary>
        private readonly ITargetManager _targetManager;

        /// <summary>
        /// Member variable _selectedTarget
        /// </summary>
        private Model.Target _selectedTarget;

        /// <summary>
        /// Member variable _targets
        /// </summary>
        private ObservableCollection<Model.Target> _targets;

        /// <summary>
        /// Member variable _editCommand
        /// </summary>
        private ICommand _editCommand;

        /// <summary>
        /// Member variable _deleteCommand
        /// </summary>
        private ICommand _deleteCommand;

        /// <summary>
        /// Member variable refreshCommand
        /// </summary>
        private ICommand _refreshCommand;

        /// <summary>
        /// Member variable _addTargetCommand
        /// </summary>
        private AddTargetCommand _addTargetCommand;

        #endregion Data Members

        #region Constructors and Initializers

        /// <summary>
        /// Initializes a new instance of the <see cref="TargetsViewModel"/> class.
        /// </summary>
        public TargetsViewModel()
            : this(ServiceLocator.Current.GetInstance<ITargetManager>())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TargetsViewModel"/> class.
        /// </summary>
        /// <param name="targetManager">The target manager.</param>
        public TargetsViewModel(ITargetManager targetManager)
        {
            Contract.Requires<ArgumentNullException>(targetManager != null);

            _targetManager = targetManager;
            Title = "Targets";
        }

        #endregion Constructors and Initializers

        #region Properties

        /// <summary>
        /// Gets or sets the targets.
        /// </summary>
        /// <value>The targets.</value>
        public ObservableCollection<Model.Target> Targets
        {
            get
            {
                if (_targets == null)
                {
                    RefreshList();
                }

                return _targets;
            }

            set
            {
                RaisePropertyChangingEvent("Targets");
                _targets = value;
                RaisePropertyChangedEvent("Targets");
            }
        }

        /// <summary>
        /// Gets or sets the selected target.
        /// </summary>
        /// <value>
        /// The selected target.
        /// </value>
        public Model.Target SelectedTarget
        {
            get
            {
                return _selectedTarget;
            }

            set
            {
                RaisePropertyChangingEvent("SelectedTarget");
                _selectedTarget = value;
                RaisePropertyChangedEvent("SelectedTarget");
            }
        }

        /// <summary>
        /// Gets the edit command.
        /// </summary>
        protected ICommand EditCommand
        {
            get
            {
                if (_editCommand == null)
                {
                    _editCommand = new DelegateCommand(EditTarget, CanEditTarget);
                }

                return _editCommand;
            }
        }

        /// <summary>
        /// Gets the delete command.
        /// </summary>
        protected ICommand DeleteCommand
        {
            get
            {
                if (_deleteCommand == null)
                {
                    _deleteCommand = new DelegateCommand(DeleteTarget, CanDeleteTarget);
                }

                return _deleteCommand;
            }
        }

        /// <summary>
        /// Gets the refresh command.
        /// </summary>
        protected ICommand RefreshCommand
        {
            get
            {
                if (_refreshCommand == null)
                {
                    _refreshCommand = new DelegateCommand(RefreshList);
                }

                return _refreshCommand;
            }
        }

        #endregion Properties

        #region Public Methods

        /// <summary>
        /// Edits the account.
        /// </summary>
        public void EditTarget()
        {
            if (SelectedTarget != null)
            {
                var regionManager = ServiceLocator.Current.GetInstance<IRegionManager>();
                var query = new UriQuery { { "ID", SelectedTarget.Id.ToString(CultureInfo.InvariantCulture) } };
                var accountEditor = new Uri(TargetModule.TargetEditor + query, UriKind.Relative);
                regionManager.RequestNavigate(Regions.SecondaryRegion, accountEditor);
            }
        }

        /// <summary>
        /// Deletes the account.
        /// </summary>
        public void DeleteTarget()
        {
            if (SelectedTarget != null)
            {
                var messageService = ServiceLocator.Current.GetInstance<IMessageBoxService>();
                var session = ServiceLocator.Current.GetInstance<ISession>();

                var result = messageService.Show(
                    session.Application.MainWindow,
                    Resources.Message_Target_Delete_Confirmation,
                    Resources.ApplicationName,
                    MessageBoxButton.YesNo,
                    MessageBoxImage.Question);
                if (result == MessageBoxResult.Yes)
                {
                    _targetManager.Delete(SelectedTarget.Map<Model.Target, Business.Entities.Target>());
                    Targets.Remove(SelectedTarget);
                }
            }
        }

        /// <summary>
        /// Determines whether this instance [can edit target].
        /// </summary>
        /// <returns>
        ///   <c>true</c> if this instance [can edit target]; otherwise, <c>false</c>.
        /// </returns>
        public bool CanEditTarget()
        {
            return true;
        }

        /// <summary>
        /// Determines whether this instance [can delete target] the specified obj.
        /// </summary>
        /// <returns>
        ///   <c>true</c> if this instance [can delete target] the specified obj; otherwise, <c>false</c>.
        /// </returns>
        public bool CanDeleteTarget()
        {
            return true;
        }

        /// <summary>
        /// Refreshes the list.
        /// </summary>
        public void RefreshList()
        {
            var data = _targetManager.List().Select(x => x.Map<Business.Entities.Target, Model.Target>());
            Targets = new ObservableCollection<Model.Target>(data.ToList());
        }

        #endregion Public Methods

        #region Implementation of INavigationAware

        /// <summary>
        /// Called when [navigated to].
        /// </summary>
        /// <param name="navigationContext">The navigation context.</param>
        public void OnNavigatedTo(NavigationContext navigationContext)
        {
            var regionManager = ServiceLocator.Current.GetInstance<IRegionManager>();
            var accountRibbonTab = new Uri(TargetModule.TargetsTabName, UriKind.Relative);
            regionManager.RequestNavigate(Regions.RibbonRegion, accountRibbonTab);

            _addTargetCommand = new AddTargetCommand();
            TargetGlobalCommands.AddTarget.RegisterCommand(_addTargetCommand);
            TargetGlobalCommands.EditTarget.RegisterCommand(EditCommand);
            TargetGlobalCommands.DeleteTarget.RegisterCommand(DeleteCommand);
            TargetGlobalCommands.RefreshTarget.RegisterCommand(RefreshCommand);
        }

        /// <summary>
        /// Determines whether [is navigation target] [the specified navigation context].
        /// </summary>
        /// <param name="navigationContext">The navigation context.</param>
        /// <returns>
        /// True if [is navigation target] [the specified navigation context]; otherwise, false.
        /// </returns>
        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            return true;
        }

        /// <summary>
        /// Called when [navigated from].
        /// </summary>
        /// <param name="navigationContext">The navigation context.</param>
        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
            TargetGlobalCommands.AddTarget.UnregisterCommand(_addTargetCommand);
            TargetGlobalCommands.EditTarget.UnregisterCommand(EditCommand);
            TargetGlobalCommands.DeleteTarget.UnregisterCommand(DeleteCommand);
            TargetGlobalCommands.RefreshTarget.UnregisterCommand(RefreshCommand);
        }

        #endregion Implementation of INavigationAware
    }
}