﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="AccountRepositoryTest.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the AccountRepositoryTest type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Data.Tests
{
    using System;
    using System.Data.Objects;
    using System.Linq;
    using Entities;
    using Interfaces;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    /// <summary>
    /// Defines the <seealso cref="Hisaab.Data.Tests.AccountRepositoryTest"/> type.
    /// </summary>
    [TestClass]
    public class AccountRepositoryTest
    {
        /// <summary>
        /// Constructors the should throw exception if database is null.
        /// </summary>
        [TestMethod, ExpectedException(typeof(ArgumentNullException))]
        public void ConstructorShouldThrowExceptionIfDatabaseIsNull()
        {
            new AccountRepository(null);
        }

        /// <summary>
        /// Constructors the should not throw exception if database is not null.
        /// </summary>
        [TestMethod]
        public void ConstructorShouldNotThrowExceptionIfDatabaseIsNotNull()
        {
            new AccountRepository(new Mock<IDatabase>().Object);
        }

        #region Account Tests

        #region Query Tests

        /// <summary>
        /// Accounts the data manager get Accounts test with empty repository.
        /// </summary>
        [TestMethod]
        public void AccountDataManagerQueryShouldReturnEmptyListWithEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            var dao = new AccountRepository(mockRepository.Object);
            var result = dao.Query(new Account(), null, null);
            Assert.IsNotNull(result);
            Assert.AreEqual(0, result.Count());
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Accounts the data manager get Accounts test with non empty repository.
        /// </summary>
        [TestMethod]
        public void AccountDataManagerQueryShouldReturnNonEmptyListWithNonEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            mockObjectSet.AddObject(new Account { Id = 1, Name = "Account 1", AccountType = 4 });
            mockObjectSet.AddObject(new Account { Id = 2, Name = "Account 2", AccountType = 3 });
            mockObjectSet.AddObject(new Account { Id = 3, Name = "Account 3", AccountType = 5 });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            var dao = new AccountRepository(mockRepository.Object);
            var result = dao.Query(new Account(), null, null);
            Assert.IsNotNull(result);
            Assert.AreEqual(3, result.Count());
            Assert.AreEqual(1, result.ElementAt(0).Id);
            Assert.AreEqual("Account 1", result.ElementAt(0).Name);
            Assert.AreEqual(2, result.ElementAt(1).Id);
            Assert.AreEqual("Account 2", result.ElementAt(1).Name);
            Assert.AreEqual(3, result.ElementAt(2).Id);
            Assert.AreEqual("Account 3", result.ElementAt(2).Name);
            mockRepository.VerifyAll();
        }

        #endregion Query Tests

        #region Filter Tests

        /// <summary>
        /// Accounts the data manager get Accounts test with empty repository.
        /// </summary>
        [TestMethod]
        public void AccountDataManagerFilterShouldReturnEmptyListWithEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            var dao = new AccountRepository(mockRepository.Object);
            var result = dao.Filter(x => true, null, null);
            Assert.IsNotNull(result);
            Assert.AreEqual(0, result.Count());
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Accounts the data manager get Accounts test with non empty repository.
        /// </summary>
        [TestMethod]
        public void AccountDataManagerFilterShouldReturnNonEmptyListWithNonEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            mockObjectSet.AddObject(new Account { Id = 1, Name = "Account 1", AccountType = 4 });
            mockObjectSet.AddObject(new Account { Id = 2, Name = "Account 2", AccountType = 3 });
            mockObjectSet.AddObject(new Account { Id = 3, Name = "Account 3", AccountType = 5 });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            var dao = new AccountRepository(mockRepository.Object);
            var result = dao.Filter(x => true, null, null);
            Assert.IsNotNull(result);
            Assert.AreEqual(3, result.Count());
            Assert.AreEqual(1, result.ElementAt(0).Id);
            Assert.AreEqual("Account 1", result.ElementAt(0).Name);
            Assert.AreEqual(2, result.ElementAt(1).Id);
            Assert.AreEqual("Account 2", result.ElementAt(1).Name);
            Assert.AreEqual(3, result.ElementAt(2).Id);
            Assert.AreEqual("Account 3", result.ElementAt(2).Name);
            mockRepository.VerifyAll();
        }

        #endregion Filter Tests

        #region GetById Tests

        /// <summary>
        /// Get by id test with empty repository
        /// </summary>
        [TestMethod]
        public void AccountDataManagerFindByIdShouldReturnNullWithEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            var dao = new AccountRepository(mockRepository.Object);
            var result = dao.FindById(1);
            Assert.IsNull(result);
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Get by id test with repository with records
        /// </summary>
        [TestMethod]
        public void AccountDataManagerFindByIdShouldReturnMatchingAccount()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            mockObjectSet.AddObject(new Account { Id = 1, Name = "Account 1", AccountType = 4 });
            mockObjectSet.AddObject(new Account { Id = 2, Name = "Account 2", AccountType = 3 });
            mockObjectSet.AddObject(new Account { Id = 3, Name = "Account 3", AccountType = 5 });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            var dao = new AccountRepository(mockRepository.Object);
            var result = dao.FindById(2);
            Assert.IsNotNull(result);
            Assert.AreEqual(2, result.Id);
            Assert.AreEqual("Account 2", result.Name);
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Accounts the data manager find by id should return null with no matching Account.
        /// </summary>
        [TestMethod]
        public void AccountDataManagerFindByIdShouldReturnNullWithNoMatchingAccount()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            mockObjectSet.AddObject(new Account { Id = 1, Name = "Account 1", AccountType = 4 });
            mockObjectSet.AddObject(new Account { Id = 2, Name = "Account 2", AccountType = 3 });
            mockObjectSet.AddObject(new Account { Id = 3, Name = "Account 3", AccountType = 5 });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            var dao = new AccountRepository(mockRepository.Object);
            var result = dao.FindById(20);
            Assert.IsNull(result);
            mockRepository.VerifyAll();
        }

        #endregion GetById Tests

        #region Insert Tests

        /// <summary>
        /// Email template dao insert test with empty repository
        /// </summary>
        [TestMethod]
        public void AccountDataManagerInsertAccountTestWithEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            mockRepository.Setup(m => m.SaveChanges());
            mockRepository.Setup(m => m.Refresh(RefreshMode.StoreWins, It.IsAny<Account>()));
            var dao = new AccountRepository(mockRepository.Object);
            var result = dao.Add(new Account { Id = 1, Name = "Account 1", AccountType = 32 });
            Assert.AreEqual(1, mockObjectSet.Count());
            Assert.IsNotNull(result);
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Email template dao insert test with existing records in repository
        /// </summary>
        [TestMethod]
        public void AccountDataManagerInsertAccountTestWithNonEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            mockObjectSet.AddObject(new Account { Id = 1, Name = "Account 1", AccountType = 32 });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            mockRepository.Setup(m => m.SaveChanges());
            mockRepository.Setup(m => m.Refresh(RefreshMode.StoreWins, It.IsAny<Account>()));
            var dao = new AccountRepository(mockRepository.Object);

            Assert.AreEqual(1, mockObjectSet.Count());

            var result = dao.Add(new Account { Id = 2, Name = "Account 2", AccountType = 43 });

            Assert.AreEqual(2, mockObjectSet.Count());
            Assert.IsNotNull(result);
            mockRepository.VerifyAll();
        }

        #endregion Insert Tests

        #region Udpate Tests

        /// <summary>
        /// Email template dao update test with no matching record
        /// </summary>
        [TestMethod]
        public void AccountDataManagerUpdateAccountTestWithNoMatchingRecord()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            mockObjectSet.AddObject(new Account { Id = 1, Name = "Account 1", AccountType = 32 });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            var dao = new AccountRepository(mockRepository.Object);
            dao.Update(new Account { Id = 2, Name = "Account 2", AccountType = 32 });

            Assert.AreEqual(1, mockObjectSet.Count());
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Email template dao update test with matching record
        /// </summary>
        [TestMethod]
        public void AccountDataManagerUpdateAccountTestWithMatchingRecord()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            mockObjectSet.AddObject(new Account { Id = 1, Name = "Account 1", AccountType = 32 });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            mockRepository.Setup(m => m.SaveChanges()).Returns(1);

            var dao = new AccountRepository(mockRepository.Object);
            dao.Update(new Account { Id = 1, Name = "Account 2", AccountType = 32 });

            Assert.AreEqual(1, mockObjectSet.Count());
            mockRepository.VerifyAll();
        }

        #endregion Udpate Tests

        #region Delete Tests

        /// <summary>
        /// Email template dao delete test with no matching record
        /// </summary>
        [TestMethod]
        public void AccountDataManagerDeleteAccountTestWithNoMatchingRecord()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            mockObjectSet.AddObject(new Account { Id = 1, Name = "Account 1", AccountType = 32 });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            var dao = new AccountRepository(mockRepository.Object);
            dao.Delete(new Account { Id = 2 });

            Assert.AreEqual(1, mockObjectSet.Count());
        }

        /// <summary>
        /// Email template dao delete test with matching record
        /// </summary>
        [TestMethod]
        public void AccountDataManagerDeleteAccountTestWithMatchingRecord()
        {
            var mockObjectSet = new MockObjectSet<Account>();
            mockObjectSet.AddObject(new Account { Id = 1, Name = "Account 1", AccountType = 323 });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Accounts).Returns(mockObjectSet);
            var dao = new AccountRepository(mockRepository.Object);
            dao.Delete(new Account { Id = 1 });

            Assert.AreEqual(0, mockObjectSet.Count());
        }

        #endregion Delete Tests

        #endregion Account Tests
    }
}