﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="AccountTypeFilterTests.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the AccountTypeFilterTests type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Data.Tests.Filters
{
    using System.Data.Objects;
    using System.Linq;
    using Data.Filters;
    using Entities;
    using Microsoft.VisualStudio.TestTools.UnitTesting;

    /// <summary>
    /// Defines the <seealso cref="Hisaab.Data.Tests.Filters.AccountTypeFilterTests"/> type.
    /// </summary>
    [TestClass]
    public class AccountTypeFilterTests
    {
        /// <summary>
        /// Mock database
        /// </summary>
        private readonly IObjectSet<AccountType> _mockObjectSet = new MockObjectSet<AccountType>();

        /// <summary>
        /// Initializes a new instance of the <see cref="AccountTypeFilterTests"/> class.
        /// </summary>
        public AccountTypeFilterTests()
        {
            _mockObjectSet.AddObject(new AccountType { Id = 1, Name = "AccountType 1" });
            _mockObjectSet.AddObject(new AccountType { Id = 2, Name = "AccountType 2" });
            _mockObjectSet.AddObject(new AccountType { Id = 3, Name = "AccountType 3" });
        }

        /// <summary>
        /// Filter_s the should return everything if the object to comare is null.
        /// </summary>
        [TestMethod]
        public void AccountTypeFilterShouldReturnEverythingIfTheObjectToComareIsNull()
        {
            var result = _mockObjectSet.Filter(null).ToList();

            Assert.IsNotNull(result);
            Assert.AreEqual(3, result.Count());
        }

        /// <summary>
        /// Filters the should return everything if the object to comare is uninitialized.
        /// </summary>
        [TestMethod]
        public void AccountTypeFilterShouldReturnEverythingIfTheObjectToComareIsUninitialized()
        {
            var result = _mockObjectSet.Filter(new AccountType()).ToList();

            Assert.IsNotNull(result);
            Assert.AreEqual(3, result.Count());
        }

        /// <summary>
        /// Filters the should return object filtered by id.
        /// </summary>
        [TestMethod]
        public void AccountTypeFilterShouldReturnObjectFilteredById()
        {
            var result = _mockObjectSet.Filter(new AccountType { Id = 2 }).ToList();

            Assert.IsNotNull(result);
            Assert.AreEqual(1, result.Count());
            Assert.AreEqual(2, result[0].Id);
            Assert.AreEqual("AccountType 2", result[0].Name);
        }

        /// <summary>
        /// Filters the name of the should return object filtered by.
        /// </summary>
        [TestMethod]
        public void AccountTypeFilterShouldReturnObjectFilteredByName()
        {
            var result = _mockObjectSet.Filter(new AccountType { Name = "AccountType 1" }).ToList();

            Assert.IsNotNull(result);
            Assert.AreEqual(1, result.Count());
            Assert.AreEqual(1, result[0].Id);
            Assert.AreEqual("AccountType 1", result[0].Name);
        }

        /// <summary>
        /// Filters the should return object filtered by name with different case.
        /// </summary>
        [TestMethod]
        public void AccountTypeAccountTypeFilterShouldReturnObjectFilteredByNameWithDifferentCase()
        {
            var result = _mockObjectSet.Filter(new AccountType { Name = "AccountType 2" }).ToList();

            Assert.IsNotNull(result);
            Assert.AreEqual(1, result.Count());
            Assert.AreEqual(2, result[0].Id);
            Assert.AreEqual("AccountType 2", result[0].Name);
        }

        /// <summary>
        /// Filters the should return object filtered by combination of fields.
        /// </summary>
        [TestMethod]
        public void AccountTypeFilterShouldReturnObjectFilteredByCombinationOfFields()
        {
            var result = _mockObjectSet.Filter(new AccountType { Id = 3, Name = "AccountType 3" }).ToList();

            Assert.IsNotNull(result);
            Assert.AreEqual(1, result.Count());
            Assert.AreEqual(3, result[0].Id);
            Assert.AreEqual("AccountType 3", result[0].Name);
        }

        /// <summary>
        /// Filters the should return nothing if mismatching object filtered by combination of fields.
        /// </summary>
        [TestMethod]
        public void AccountTypeFilterShouldReturnNothingIfMismatchingObjectFilteredByCombinationOfFields()
        {
            var result = _mockObjectSet.Filter(new AccountType { Id = 3, Name = "AccountType 2" }).ToList();

            Assert.IsNotNull(result);
            Assert.AreEqual(0, result.Count());
        }
    }
}