﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="TransactionRepositoryTest.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the TransactionRepositoryTest type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Data.Tests
{
    using System.Data.Objects;
    using System.Linq;
    using Entities;
    using Interfaces;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    /// <summary>
    /// Defines the <seealso cref="Hisaab.Data.Tests.TransactionRepositoryTest"/> type.
    /// </summary>
    [TestClass]
    public class TransactionRepositoryTest
    {
        #region Transaction Tests

        #region Query Tests

        /// <summary>
        /// Transactions the data manager get Transactions test with empty repository.
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerQueryShouldReturnEmptyListWithEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            var dao = new TransactionRepository(mockRepository.Object);
            var result = dao.Query(new Transaction(), null, null);
            Assert.IsNotNull(result);
            Assert.AreEqual(0, result.Count());
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Transactions the data manager get Transactions test with non empty repository.
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerQueryShouldReturnNonEmptyListWithNonEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            mockObjectSet.AddObject(new Transaction { Id = 1, Title = "Transaction 1" });
            mockObjectSet.AddObject(new Transaction { Id = 2, Title = "Transaction 2" });
            mockObjectSet.AddObject(new Transaction { Id = 3, Title = "Transaction 3" });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            var dao = new TransactionRepository(mockRepository.Object);
            var result = dao.Query(new Transaction(), null, null);
            Assert.IsNotNull(result);
            Assert.AreEqual(3, result.Count());
            Assert.AreEqual(1, result.ElementAt(0).Id);
            Assert.AreEqual("Transaction 1", result.ElementAt(0).Title);
            Assert.AreEqual(2, result.ElementAt(1).Id);
            Assert.AreEqual("Transaction 2", result.ElementAt(1).Title);
            Assert.AreEqual(3, result.ElementAt(2).Id);
            Assert.AreEqual("Transaction 3", result.ElementAt(2).Title);
            mockRepository.VerifyAll();
        }

        #endregion Query Tests

        #region Filter Tests

        /// <summary>
        /// Transactions the data manager get Transactions test with empty repository.
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerFilterShouldReturnEmptyListWithEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            var dao = new TransactionRepository(mockRepository.Object);
            var result = dao.Filter(x => true, null, null);
            Assert.IsNotNull(result);
            Assert.AreEqual(0, result.Count());
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Transactions the data manager get Transactions test with non empty repository.
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerFilterShouldReturnNonEmptyListWithNonEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            mockObjectSet.AddObject(new Transaction { Id = 1, Title = "Transaction 1" });
            mockObjectSet.AddObject(new Transaction { Id = 2, Title = "Transaction 2" });
            mockObjectSet.AddObject(new Transaction { Id = 3, Title = "Transaction 3" });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            var dao = new TransactionRepository(mockRepository.Object);
            var result = dao.Filter(x => true, null, null);
            Assert.IsNotNull(result);
            Assert.AreEqual(3, result.Count());
            Assert.AreEqual(1, result.ElementAt(0).Id);
            Assert.AreEqual("Transaction 1", result.ElementAt(0).Title);
            Assert.AreEqual(2, result.ElementAt(1).Id);
            Assert.AreEqual("Transaction 2", result.ElementAt(1).Title);
            Assert.AreEqual(3, result.ElementAt(2).Id);
            Assert.AreEqual("Transaction 3", result.ElementAt(2).Title);
            mockRepository.VerifyAll();
        }

        #endregion Filter Tests

        #region GetById Tests

        /// <summary>
        /// Get by id test with empty repository
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerFindByIdShouldReturnNullWithEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            var dao = new TransactionRepository(mockRepository.Object);
            var result = dao.FindById(1);
            Assert.IsNull(result);
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Get by id test with repository with records
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerFindByIdShouldReturnMatchingTransaction()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            mockObjectSet.AddObject(new Transaction { Id = 1, Title = "Transaction 1" });
            mockObjectSet.AddObject(new Transaction { Id = 2, Title = "Transaction 2" });
            mockObjectSet.AddObject(new Transaction { Id = 3, Title = "Transaction 3" });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            var dao = new TransactionRepository(mockRepository.Object);
            var result = dao.FindById(2);
            Assert.IsNotNull(result);
            Assert.AreEqual(2, result.Id);
            Assert.AreEqual("Transaction 2", result.Title);
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Transactions the data manager find by id should return null with no matching Transaction.
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerFindByIdShouldReturnNullWithNoMatchingTransaction()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            mockObjectSet.AddObject(new Transaction { Id = 1, Title = "Transaction 1" });
            mockObjectSet.AddObject(new Transaction { Id = 2, Title = "Transaction 2" });
            mockObjectSet.AddObject(new Transaction { Id = 3, Title = "Transaction 3" });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            var dao = new TransactionRepository(mockRepository.Object);
            var result = dao.FindById(20);
            Assert.IsNull(result);
            mockRepository.VerifyAll();
        }

        #endregion GetById Tests

        #region Insert Tests

        /// <summary>
        /// Email template dao insert test with empty repository
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerInsertTransactionTestWithEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            mockRepository.Setup(m => m.SaveChanges());
            mockRepository.Setup(m => m.Refresh(RefreshMode.StoreWins, It.IsAny<Transaction>()));
            var dao = new TransactionRepository(mockRepository.Object);
            var result = dao.Add(new Transaction { Id = 1, Title = "Transaction 1" });
            Assert.AreEqual(1, mockObjectSet.Count());
            Assert.IsNotNull(result);
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Email template dao insert test with existing records in repository
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerInsertTransactionTestWithNonEmptyRepository()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            mockObjectSet.AddObject(new Transaction { Id = 1, Title = "Transaction 1" });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            mockRepository.Setup(m => m.SaveChanges());
            mockRepository.Setup(m => m.Refresh(RefreshMode.StoreWins, It.IsAny<Transaction>()));
            var dao = new TransactionRepository(mockRepository.Object);

            Assert.AreEqual(1, mockObjectSet.Count());

            var result = dao.Add(new Transaction { Id = 2, Title = "Transaction 2" });

            Assert.AreEqual(2, mockObjectSet.Count());
            Assert.IsNotNull(result);
            mockRepository.VerifyAll();
        }

        #endregion Insert Tests

        #region Udpate Tests

        /// <summary>
        /// Email template dao update test with no matching record
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerUpdateTransactionTestWithNoMatchingRecord()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            mockObjectSet.AddObject(new Transaction { Id = 1, Title = "Transaction 1" });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            var dao = new TransactionRepository(mockRepository.Object);
            dao.Update(new Transaction { Id = 2, Title = "Transaction 2" });

            Assert.AreEqual(1, mockObjectSet.Count());
            mockRepository.VerifyAll();
        }

        /// <summary>
        /// Email template dao update test with matching record
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerUpdateTransactionTestWithMatchingRecord()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            mockObjectSet.AddObject(new Transaction { Id = 1, Title = "Transaction 1" });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            mockRepository.Setup(m => m.SaveChanges()).Returns(1);

            var dao = new TransactionRepository(mockRepository.Object);
            dao.Update(new Transaction { Id = 1, Title = "Transaction 2" });

            Assert.AreEqual(1, mockObjectSet.Count());
            mockRepository.VerifyAll();
        }

        #endregion Udpate Tests

        #region Delete Tests

        /// <summary>
        /// Email template dao delete test with no matching record
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerDeleteTransactionTestWithNoMatchingRecord()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            mockObjectSet.AddObject(new Transaction { Id = 1, Title = "Transaction 1" });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            var dao = new TransactionRepository(mockRepository.Object);
            dao.Delete(new Transaction { Id = 2 });

            Assert.AreEqual(1, mockObjectSet.Count());
        }

        /// <summary>
        /// Email template dao delete test with matching record
        /// </summary>
        [TestMethod]
        public void TransactionDataManagerDeleteTransactionTestWithMatchingRecord()
        {
            var mockObjectSet = new MockObjectSet<Transaction>();
            mockObjectSet.AddObject(new Transaction { Id = 1, Title = "Transaction 1" });
            var mockRepository = new Mock<IDatabase>();
            mockRepository.SetupGet(m => m.Transactions).Returns(mockObjectSet);
            var dao = new TransactionRepository(mockRepository.Object);
            dao.Delete(new Transaction { Id = 1 });

            Assert.AreEqual(0, mockObjectSet.Count());
        }

        #endregion Delete Tests

        #endregion Transaction Tests
    }
}