﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="TagRepository.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the <seealso cref="Hisaab.Data.TagRepository" /> type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Data
{
    using System;
    using System.Collections.Generic;
    using System.Data.Objects;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Linq.Expressions;
    using Entities;
    using Filters;
    using Interfaces;

    using Microsoft.Practices.ServiceLocation;

    /// <summary>
    /// Defines the <seealso cref="Hisaab.Data.TagRepository"/> type.
    /// </summary>
    public class TagRepository : ITagRepository
    {
        /// <summary>
        /// Databse instance to use
        /// </summary>
        private readonly IDatabase _database;

        /// <summary>
        /// Initializes a new instance of the <see cref="TagRepository"/> class.
        /// </summary>
        public TagRepository()
            : this(ServiceLocator.Current.GetInstance<IDatabase>())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TagRepository"/> class.
        /// </summary>
        /// <param name="database">The database.</param>
        public TagRepository(IDatabase database)
        {
            Contract.Requires<ArgumentNullException>(database != null);
            _database = database;
        }

        /// <summary>
        /// Gets the filtered.
        /// </summary>
        /// <param name="filter">The filter.</param>
        /// <param name="pageNo">The page no.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <returns>List of filtered items</returns>
        public IQueryable<Tag> Query(Tag filter, int? pageNo, int? pageSize)
        {
            return _database.Tags
                .Filter(filter)
                .Page(pageNo, pageSize);
        }

        /// <summary>
        /// Queries the specified filter.
        /// </summary>
        /// <param name="filter">The filter.</param>
        /// <param name="pageNo">The page no.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <returns>List of filtered items</returns>
        public IQueryable<Tag> Filter(Expression<Func<Tag, bool>> filter, int? pageNo, int? pageSize)
        {
            return _database.Tags
                .Where(filter)
                .Page(pageNo, pageSize);
        }

        /// <summary>
        /// Finds the by id.
        /// </summary>
        /// <param name="id">The id of entity</param>
        /// <returns>Entity matching by id</returns>
        public Tag FindById(int id)
        {
            return _database.Tags
                .Filter(new Tag { Id = id })
                .FirstOrDefault();
        }

        /// <summary>
        /// Adds the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        /// <returns>Id of tag added</returns>
        public int Add(Tag entity)
        {
            _database.Tags.AddObject(entity);
            _database.SaveChanges();
            _database.Refresh(RefreshMode.StoreWins, entity);
            return entity.Id;
        }

        /// <summary>
        /// Updates the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        public void Update(Tag entity)
        {
            var previousEntity = _database.Tags.Filter(new Tag { Id = entity.Id }).FirstOrDefault();

            if (previousEntity != null)
            {
                previousEntity.Name = entity.Name;
            }

            _database.SaveChanges();
        }

        /// <summary>
        /// Deletes the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        public void Delete(Tag entity)
        {
            var previousEntity = _database.Tags.Filter(new Tag { Id = entity.Id }).FirstOrDefault();

            if (previousEntity != null)
            {
                _database.Tags.DeleteObject(previousEntity);
            }

            _database.SaveChanges();
        }

        /// <summary>
        /// Gets the tags for transaction.
        /// </summary>
        /// <param name="transactionId">The transaction id.</param>
        /// <returns>
        /// List of tags associated with given transaction id
        /// </returns>
        public IEnumerable<Tag> GetTagsForTransaction(int transactionId)
        {
            return _database.Tags.FilterByTransaction(transactionId);
        }
    }
}