﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="TargetRepository.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the <seealso cref="TargetRepository" /> type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Data
{
    using System;
    using System.Data.Objects;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Linq.Expressions;

    using Hisaab.Data.Entities;
    using Hisaab.Data.Filters;
    using Hisaab.Data.Interfaces;

    using Microsoft.Practices.ServiceLocation;

    /// <summary>
    ///  Defines the <seealso cref="TargetRepository"/> type.
    /// </summary>
    public class TargetRepository : ITargetRepository
    {
        /// <summary>
        /// Databse instance to use
        /// </summary>
        private readonly IDatabase _database;

        /// <summary>
        /// Initializes a new instance of the <see cref="TargetRepository"/> class.
        /// </summary>
        public TargetRepository()
            : this(ServiceLocator.Current.GetInstance<IDatabase>())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TargetRepository"/> class.
        /// </summary>
        /// <param name="database">The database.</param>
        public TargetRepository(IDatabase database)
        {
            Contract.Requires<ArgumentNullException>(database != null);
            _database = database;
        }

        /// <summary>
        /// Gets the filtered.
        /// </summary>
        /// <param name="filter">The filter.</param>
        /// <param name="pageNo">The page no.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <returns>List of filtered items</returns>
        public IQueryable<Target> Query(Target filter, int? pageNo = new int?(), int? pageSize = new int?())
        {
            return _database.Targets
                .Filter(filter)
                .Page(pageNo, pageSize);
        }

        /// <summary>
        /// Queries the specified filter.
        /// </summary>
        /// <param name="filter">The filter.</param>
        /// <param name="pageNo">The page no.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <returns>List of filtered items</returns>
        public IQueryable<Target> Filter(Expression<Func<Target, bool>> filter, int? pageNo = new int?(), int? pageSize = new int?())
        {
            return _database.Targets
                .Where(filter)
                .Page(pageNo, pageSize);
        }

        /// <summary>
        /// Finds the by id.
        /// </summary>
        /// <param name="id">The id of entity</param>
        /// <returns>Entity matching by id</returns>
        public Target FindById(int id)
        {
            return _database.Targets
                .Filter(new Target { Id = id })
                .FirstOrDefault();
        }

        /// <summary>
        /// Adds the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        /// <returns>Id of entity added</returns>
        public int Add(Target entity)
        {
            _database.Targets.AddObject(entity);
            _database.SaveChanges();
            _database.Refresh(RefreshMode.StoreWins, entity);
            return entity.Id;
        }

        /// <summary>
        /// Updates the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        public void Update(Target entity)
        {
            var previousEntity = _database.Targets.Filter(new Target { Id = entity.Id }).FirstOrDefault();

            if (previousEntity != null)
            {
                previousEntity.Title = entity.Title;
                previousEntity.TargetType = entity.TargetType;
                previousEntity.TargetDate = entity.TargetDate;
                previousEntity.TagId = entity.TagId;
                previousEntity.AccountId = entity.AccountId;
                previousEntity.Amount = entity.Amount;
            }

            _database.SaveChanges();
        }

        /// <summary>
        /// Deletes the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        public void Delete(Target entity)
        {
            var previousEntity = _database.Targets.Filter(new Target { Id = entity.Id }).FirstOrDefault();

            if (previousEntity != null)
            {
                _database.Targets.DeleteObject(previousEntity);
            }

            _database.SaveChanges();
        }
    }
}