﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="VehicleRepository.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the <seealso cref="Hisaab.Data.VehicleRepository" /> type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Data
{
    using System;
    using System.Data.Objects;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Linq.Expressions;
    using Entities;
    using Filters;
    using Interfaces;

    using Microsoft.Practices.ServiceLocation;

    /// <summary>
    /// Defines the <seealso cref="Hisaab.Data.VehicleRepository"/> type.
    /// </summary>
    public class VehicleRepository : IVehicleRepository
    {
        /// <summary>
        /// Databse instance to use
        /// </summary>
        private readonly IDatabase _database;

        /// <summary>
        /// Initializes a new instance of the <see cref="VehicleRepository"/> class.
        /// </summary>
        public VehicleRepository()
            : this(ServiceLocator.Current.GetInstance<IDatabase>())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="VehicleRepository"/> class.
        /// </summary>
        /// <param name="database">The database.</param>
        public VehicleRepository(IDatabase database)
        {
            Contract.Requires<ArgumentNullException>(database != null);
            _database = database;
        }

        /// <summary>
        /// Gets the filtered.
        /// </summary>
        /// <param name="filter">The filter.</param>
        /// <param name="pageNo">The page no.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <returns>List of filtered items</returns>
        public IQueryable<Vehicle> Query(Vehicle filter, int? pageNo, int? pageSize)
        {
            return _database.Vehicles
                .Filter(filter)
                .Page(pageNo, pageSize);
        }

        /// <summary>
        /// Queries the specified filter.
        /// </summary>
        /// <param name="filter">The filter.</param>
        /// <param name="pageNo">The page no.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <returns>List of filtered items</returns>
        public IQueryable<Vehicle> Filter(Expression<Func<Vehicle, bool>> filter, int? pageNo, int? pageSize)
        {
            return _database.Vehicles
                .Where(filter)
                .Page(pageNo, pageSize);
        }

        /// <summary>
        /// Finds the by id.
        /// </summary>
        /// <param name="id">The id of entity</param>
        /// <returns>Entity matching by id</returns>
        public Vehicle FindById(int id)
        {
            return _database.Vehicles
                .Filter(new Vehicle { Id = id })
                .FirstOrDefault();
        }

        /// <summary>
        /// Adds the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        /// <returns>Id of vehicle added</returns>
        public int Add(Vehicle entity)
        {
            _database.Vehicles.AddObject(entity);
            _database.SaveChanges();
            _database.Refresh(RefreshMode.StoreWins, entity);
            return entity.Id;
        }

        /// <summary>
        /// Updates the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        public void Update(Vehicle entity)
        {
            var previousEntity = _database.Vehicles.Filter(new Vehicle { Id = entity.Id }).FirstOrDefault();

            if (previousEntity != null)
            {
                previousEntity.Id = entity.Id;
                previousEntity.Title = entity.Title;
            }

            _database.SaveChanges();
        }

        /// <summary>
        /// Deletes the specified entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        public void Delete(Vehicle entity)
        {
            var previousEntity = _database.Vehicles.Filter(new Vehicle { Id = entity.Id }).FirstOrDefault();

            if (previousEntity != null)
            {
                _database.Vehicles.DeleteObject(previousEntity);
            }

            _database.SaveChanges();
        }
    }
}