﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="AccountsViewModel.cs" company="Concept Architecture">
//   (c) 2010 Muhammad Umer Farooq
// </copyright>
// <summary>
//   Defines the AccountViewModel type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Hisaab.Windows.Client.ViewModel
{
    using System;
    using System.Collections.ObjectModel;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Windows.Forms;
    using Business.Interfaces;
    using Common;
    using Documents;
    using Hisaab.Common;

    using Microsoft.Practices.ServiceLocation;

    using Model;

    /// <summary>
    /// Defines the <seealso cref="AccountsViewModel"/> type.
    /// </summary>
    public class AccountsViewModel : ViewModelBase
    {
        /// <summary>
        /// The Save Command
        /// </summary>
        public static readonly ActiveAwareCommand EditCommand = new ActiveAwareCommand();

        /// <summary>
        /// The Delete Command
        /// </summary>
        public static readonly ActiveAwareCommand DeleteCommand = new ActiveAwareCommand();

        /// <summary>
        /// The refresh command
        /// </summary>
        public static readonly ActiveAwareCommand RefreshCommand = new ActiveAwareCommand();

        /// <summary>
        /// Session manager
        /// </summary>
        private readonly ISession _session;

        /// <summary>
        /// Account object
        /// </summary>
        private readonly IAccountManager _accountManager;

        /// <summary>
        /// Summary view model
        /// </summary>
        private AccountSummaryViewModel _summaryViewModel;

        /// <summary>
        /// Selected account
        /// </summary>
        private Account _selectedAccount;

        /// <summary>
        /// Accounts list
        /// </summary>
        private ObservableCollection<Account> _accounts;

        /// <summary>
        /// Initializes a new instance of the <see cref="AccountsViewModel"/> class.
        /// </summary>
        public AccountsViewModel()
            : this(ServiceLocator.Current.GetInstance<IAccountManager>(), ServiceLocator.Current.GetInstance<ISession>())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AccountsViewModel"/> class.
        /// </summary>
        /// <param name="accountManager">The account.</param>
        /// <param name="session">The session</param>
        public AccountsViewModel(IAccountManager accountManager, ISession session)
        {
            Contract.Requires<ArgumentNullException>(accountManager != null);
            Contract.Requires<ArgumentNullException>(session != null);

            _accountManager = accountManager;
            _session = session;
        }

        /// <summary>
        /// Gets the accounts.
        /// </summary>
        public ObservableCollection<Account> Accounts
        {
            get
            {
                if (_accounts == null)
                {
                    _accounts = new ObservableCollection<Account>(
                        _accountManager
                        .List(new Business.Entities.Account { UserId = _session.LoggedUser })
                        .Select(x => x.Map<Business.Entities.Account, Account>()));
                }

                return _accounts;
            }

            private set
            {
                if (value != _accounts)
                {
                    _accounts = value;

                    OnPropertyChanged("Accounts");
                }
            }
        }

        /// <summary>
        /// Gets or sets the selected account.
        /// </summary>
        /// <value>
        /// The selected account.
        /// </value>
        public Account SelectedAccount
        {
            get
            {
                return _selectedAccount;
            }

            set
            {
                if (value != _selectedAccount)
                {
                    _selectedAccount = value;
                    OnPropertyChanged("SelectedAccount");
                    SummaryViewModel = new AccountSummaryViewModel(value);
                }
            }
        }

        /// <summary>
        /// Gets or sets the summary view model.
        /// </summary>
        /// <value>
        /// The summary view model.
        /// </value>
        public AccountSummaryViewModel SummaryViewModel
        {
            get
            {
                return _summaryViewModel;
            }

            set
            {
                if (_summaryViewModel != value)
                {
                    _summaryViewModel = value;
                    OnPropertyChanged("SummaryViewModel");
                }
            }
        }

        /// <summary>
        /// Gets or sets the selected account view model.
        /// </summary>
        /// <value>
        /// The selected account view model.
        /// </value>
        public AccountSummaryViewModel SelectedAccountViewModel { get; set; }

        /// <summary>
        /// Refreshes the list.
        /// </summary>
        /// <param name="obj">The object</param>
        public void RefreshList(object obj)
        {
            Accounts = new ObservableCollection<Account>(
                _accountManager
                    .List(new Business.Entities.Account { UserId = _session.LoggedUser })
                    .Select(x => x.Map<Business.Entities.Account, Account>()));
        }

        /// <summary>
        /// Edits the account.
        /// </summary>
        /// <param name="obj">The object</param>
        public void EditAccount(object obj)
        {
            var viewModel = new AccountEditorViewModel(SelectedAccount);
            var accountsDocument = new AccountEditorDocument(viewModel);

            DocumentManager.Instance.ActiveDocument = accountsDocument;
        }

        /// <summary>
        /// Deletes the account.
        /// </summary>
        /// <param name="obj">The object</param>
        public void DeleteAccount(object obj)
        {
            if (ServiceLocator.Current.GetInstance<IWindowDialogService>().ShowMessage(
                   "Are you sure you want to delete selected account?",
                   "Hissab",
                   MessageBoxButtons.YesNo,
                   MessageBoxIcon.Question)
                   == DialogResult.Yes)
            {
                _accountManager.Delete(SelectedAccount.Map<Account, Business.Entities.Account>());
                Accounts.Remove(SelectedAccount);
            }
        }

        /// <summary>
        /// Determines whether this instance [can edit tag].
        /// </summary>
        /// <param name="obj">The object</param>
        /// <returns>
        ///   <c>true</c> if this instance [can edit tag]; otherwise, <c>false</c>.
        /// </returns>
        public bool CanEditAccount(object obj)
        {
            return SelectedAccount != null;
        }

        /// <summary>
        /// Determines whether this instance [can delete tag] the specified obj.
        /// </summary>
        /// <param name="obj">The object</param>
        /// <returns>
        ///   <c>true</c> if this instance [can delete tag] the specified obj; otherwise, <c>false</c>.
        /// </returns>
        public bool CanDeleteAccount(object obj)
        {
            return SelectedAccount != null;
        }
    }
}